<?php

namespace Drupal\files_and_folders\Access;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Session\AccountInterface;
use Drupal\node\NodeInterface;

class FilesAndFoldersAccessChecker {
  protected $currentUser;

  public function __construct(AccountInterface $current_user) {
    $this->currentUser = $current_user;
  }

  public function access(NodeInterface $node = null) {
    // Si aucun nœud n'est spécifié, permettre l'accès (pour la vue générale)
    if (!$node) {
      return AccessResult::allowed();
    }
    
    // Les administrateurs ont toujours accès
    if ($this->currentUser->hasPermission('administer files and folders')) {
      return AccessResult::allowed();
    }
    
    // Vérifier si le nœud est un dossier ou un fichier
    if ($node->getType() == 'folder' || $node->getType() == 'file') {
      // Si le nœud est public, permettre l'accès
      if ($node->hasField('field_visibility') && $node->get('field_visibility')->value == 'public') {
        return AccessResult::allowed();
      }
      
      // Vérifier si l'utilisateur a la permission de voir les fichiers/dossiers privés
      if ($this->currentUser->hasPermission('view private files and folders')) {
        return AccessResult::allowed();
      }
    }
    
    // Par défaut, interdire l'accès
    return AccessResult::forbidden();
  }
}