<?php

namespace Drupal\files_and_folders\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;
use Drupal\node\Entity\Node;

class FileForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'file_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $file = NULL) {
    $entity = $file ? Node::load($file) : Node::create(['type' => 'file']);

    $form['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('File Name'),
      '#required' => TRUE,
      '#default_value' => $entity->label(),
    ];

    $form['field_file'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('File'),
      '#upload_location' => 'public://files_and_folders/',
      '#required' => TRUE,
      '#default_value' => $entity->get('field_file')->target_id ? [$entity->get('field_file')->target_id] : NULL,
    ];

    $form['field_parent_folder'] = [
      '#type' => 'entity_autocomplete',
      '#title' => $this->t('Parent Folder'),
      '#target_type' => 'node',
      '#selection_settings' => [
        'target_bundles' => ['folder'],
      ],
      '#default_value' => $entity->get('field_parent_folder')->entity,
      '#required' => FALSE,
    ];

    $form['field_visibility'] = [
      '#type' => 'select',
      '#title' => $this->t('Visibility'),
      '#options' => [
        'public' => $this->t('Public'),
        'private' => $this->t('Private'),
        'restricted' => $this->t('Restricted'),
      ],
      '#default_value' => $entity->get('field_visibility')->value,
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    ];

    $form['file_id'] = [
      '#type' => 'hidden',
      '#value' => $file,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Add validation logic if needed.
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $file_id = $form_state->getValue('file_id');
    $file_node = $file_id ? Node::load($file_id) : Node::create(['type' => 'file']);

    $file_node->setTitle($form_state->getValue('name'));
    $file_node->set('field_visibility', $form_state->getValue('field_visibility'));
    $file_node->set('field_parent_folder', $form_state->getValue('field_parent_folder'));

    $file = File::load($form_state->getValue('field_file')[0]);
    if ($file) {
      $file_node->set('field_file', $file);
      $file_node->set('field_file_size', $file->getSize());
      $file_node->set('field_file_type', $file->getMimeType());
    }

    $file_node->save();

    $this->messenger()->addMessage($this->t('File saved successfully.'));
    $form_state->setRedirect('files_and_folders.content', ['folder' => $file_node->get('field_parent_folder')->target_id]);
  }
}