<?php

namespace Drupal\files_and_folders\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\node\Entity\Node;

class FolderForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'folder_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $folder = NULL) {
    $entity = $folder ? Node::load($folder) : Node::create(['type' => 'folder']);

    $form['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Folder Name'),
      '#required' => TRUE,
      '#default_value' => $entity->label(),
    ];

    $form['field_parent_folder'] = [
      '#type' => 'entity_autocomplete',
      '#title' => $this->t('Parent Folder'),
      '#target_type' => 'node',
      '#selection_settings' => [
        'target_bundles' => ['folder'],
      ],
      '#default_value' => $entity->get('field_parent_folder')->entity,
      '#required' => FALSE,
    ];

    $form['field_visibility'] = [
      '#type' => 'select',
      '#title' => $this->t('Visibility'),
      '#options' => [
        'public' => $this->t('Public'),
        'private' => $this->t('Private'),
        'restricted' => $this->t('Restricted'),
      ],
      '#default_value' => $entity->get('field_visibility')->value,
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    ];

    $form['folder_id'] = [
      '#type' => 'hidden',
      '#value' => $folder,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Add validation logic if needed.
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $folder_id = $form_state->getValue('folder_id');
    $folder = $folder_id ? Node::load($folder_id) : Node::create(['type' => 'folder']);

    $folder->setTitle($form_state->getValue('name'));
    $folder->set('field_visibility', $form_state->getValue('field_visibility'));
    $folder->set('field_parent_folder', $form_state->getValue('field_parent_folder'));

    $folder->save();

    $this->messenger()->addMessage($this->t('Folder saved successfully.'));
    $form_state->setRedirect('files_and_folders.content', ['folder' => $folder->id()]);
  }
}